@@ -261,7 +261,7 @@ class BaseShell(object):
          existing connection as the default one.
         """
 
-    def execute(self, command, connection=None):
+    def execute(self, command, *args, connection=None, **kwargs):
         """
         Executes a command.
 
@@ -276,11 +276,11 @@ class BaseShell(object):
         :rtype: str
         :return: Shell response to the command being sent.
         """
-        self.send_command(command, connection=connection)
+        self.send_command(command, *args, connection=connection, **kwargs)
         return self.get_response(connection=connection)
 
-    def __call__(self, command, connection=None):
-        return self.execute(command, connection=connection)
+    def __call__(self, command, *args, connection=None, **kwargs):
+        return self.execute(command, *args, connection=connection, **kwargs)
 
     def _setup_shell(self, *args, connection=None, **kwargs):
         """
@@ -386,6 +386,11 @@ class PExpectShell(BaseShell):
         self._password = password
         self._password_match = password_match
         self._prefix = prefix
+        # if no timeout is provided use "the default value specified by the
+        # class timeout attribute"
+        # https://pexpect.readthedocs.io/en/stable/api/pexpect.html#pexpect.spawn.expect
+        # which is 30s
+        # https://pexpect.readthedocs.io/en/stable/api/pexpect.html#pexpect.spawn
         self._timeout = timeout or -1
         self._encoding = encoding
         self._try_filter_echo = try_filter_echo
@@ -395,13 +400,16 @@ class PExpectShell(BaseShell):
         self._node_identifier = None
         self._shell_name = None
         self._errors = errors
-
-        # Doing this to avoid having a mutable object as default value in the
-        # arguments.
-        if spawn_args is None:
-            self._spawn_args = {'env': {'TERM': 'dumb'}, 'echo': False}
-        else:
-            self._spawn_args = spawn_args
+        self._testlog = None
+        self._spawn_args = {
+            'env': {'TERM': 'dumb'},
+            'echo': False,
+            # The use_poll attribute enables using select.poll() over
+            # select.select() for socket handling. This is handy if your system
+            # could have > 1024 fds
+            'use_poll': True,
+            **(spawn_args or {})
+        }
 
         self._last_command = None
 
@@ -508,9 +516,14 @@ class PExpectShell(BaseShell):
 
         # Log log_send_command
         if not silent:
-            spawn._connection_logger.log_send_command(
-                command, matches, newline, timeout
-            )
+            if self._testlog:
+                self._testlog.log_send_command(
+                    self._node_identifier, self._shell_name, command,
+                    None if matches == [self._prompt] else matches, timeout)
+            else:
+                spawn._connection_logger.log_send_command(
+                    command, matches, newline, timeout
+                )
 
         # Expect matches
         if timeout is None:
@@ -549,9 +562,12 @@ class PExpectShell(BaseShell):
 
         # Remove leading and trailing whitespaces and normalize newlines
         text = text.strip().replace('\r', '')
+        term_codes_regex = getattr(
+            self, 'TERM_CODES_REGEX', TERM_CODES_REGEX
+        )
 
         # Remove control codes
-        text = regex_sub(TERM_CODES_REGEX, '', text)
+        text = regex_sub(term_codes_regex, '', text)
 
         # Split text into lines
         lines = text.splitlines()
@@ -569,7 +585,11 @@ class PExpectShell(BaseShell):
 
         # Log response
         if not silent:
-            spawn._connection_logger.log_get_response(response)
+            if self._testlog:
+                self._testlog.log_get_response(
+                    self._node_identifier, self._shell_name, response)
+            else:
+                spawn._connection_logger.log_get_response(response)
 
         return response
 
@@ -733,7 +753,7 @@ class PExpectBashShell(PExpectShell):
 
     def __init__(
             self,
-            initial_prompt=['\w+@.+:.+[#$] ', FORCED_PROMPT],
+            initial_prompt=[r'\w+@.+:.+[#$] ', FORCED_PROMPT],
             try_filter_echo=False, delay_after_echo_off=1, **kwargs):
 
         self._delay_after_echo_off = delay_after_echo_off
