@@ -39,17 +39,21 @@ For reference see:
     http://pytest.org/dev/plugins.html#hook-specification-and-validation
 """
 
+from time import time
+from logging import getLogger
 from os import getcwd, makedirs
 from traceback import format_exc
 from collections import OrderedDict
-from os.path import join, isabs, abspath, exists, isdir
-
 from pytest import fixture, fail, hookimpl, skip
+from os.path import join, isabs, abspath, realpath, exists, isdir
 
-from topology.args import parse_options
+from topology.args import parse_options, ExtendAction
 from topology.logging import get_logger, StepLogger
 
 
+log = getLogger(__name__)
+
+
 class TopologyPlugin(object):
     """
     pytest plugin for Topology.
@@ -60,11 +64,20 @@ class TopologyPlugin(object):
     :param str plot_format: Format to plot the topologies.
     :param str nml_dir: Directory to auto-export topologies. ``None`` if
      feature is disabled.
+    :param dict injected_attr: A dictionary holding topology attributes to
+     inject.
+    :param str log_dir: Path where to store logs.
+    :param list szn_dir: List of paths to directories where ``*.szn`` files
+     are located.
+    :param dict platform_options: Dictionary holding parameters passed directly
+     to the topology platform object.
+    :param int build_retries: Amount of times to retry the build stage.
     """
 
     def __init__(
         self, platform, plot_dir, plot_format,
-        nml_dir, injected_attr, log_dir, platform_options
+        nml_dir, injected_attr, log_dir, szn_dir, platform_options,
+        build_retries
     ):
         super(TopologyPlugin, self).__init__()
         self.platform = platform
@@ -73,7 +86,9 @@ class TopologyPlugin(object):
         self.nml_dir = nml_dir
         self.injected_attr = injected_attr
         self.log_dir = log_dir
+        self.szn_dir = szn_dir
         self.platform_options = platform_options
+        self.build_retries = build_retries
 
     def pytest_report_header(self, config):
         """
@@ -167,8 +182,31 @@ def topology(request):
                 topomgr.load(topo, inject=suite_injected_attr)
             else:
                 topomgr.parse(topo, inject=suite_injected_attr)
-            topomgr.build()
         except Exception:
+            fail(
+                'Error loading topology in module {}:\n{}'.format(
+                    module.__name__,
+                    format_exc()
+                ),
+                pytrace=False
+            )
+
+        for iteration in range(plugin.build_retries + 1):
+            try:
+                topomgr.build()
+                log.info(
+                    'Attempt {} on building topology was successful'.format(
+                        iteration
+                    )
+                )
+                break
+            except Exception:
+                msg = (
+                    '{}\nAttempt {} to build topology failed.'
+                ).format(format_exc(), iteration)
+
+                log.warning(msg)
+        else:
             fail(
                 'Error building topology in module {}:\n{}'.format(
                     module.__name__,
@@ -233,19 +271,34 @@ def pytest_addoption(parser):
         default=None,
         help='Path to a directory where logs are to be stored'
     )
+    group.addoption(
+        '--topology-szn-dir',
+        default=None,
+        action='append',
+        help='Path to a directory where szn files are located. '
+             'Can be used multiple times'
+    )
     group.addoption(
         '--topology-platform-options',
         nargs='+',
         default=None,
+        action=ExtendAction,
         help='An argument used by the topology platform '
              'with the form <key>=<value>'
     )
+    group.addoption(
+        '--topology-build-retries',
+        default=0,
+        type='int',
+        help='Retry building a topology up to defined times'
+    )
 
 
-def pytest_configure(config):
+def pytest_sessionstart(session):
     """
     pytest hook to configure plugin.
     """
+    config = session.config
     # Get registered options
     platform = config.getoption('--topology-platform')
     plot_format = config.getoption('--topology-plot-format')
@@ -253,7 +306,12 @@ def pytest_configure(config):
     nml_dir = config.getoption('--topology-nml-dir')
     injection_file = config.getoption('--topology-inject')
     log_dir = config.getoption('--topology-log-dir')
+    szn_dir = config.getoption('--topology-szn-dir')
     platform_options = config.getoption('--topology-platform-options')
+    build_retries = config.getoption('--topology-build-retries')
+
+    if build_retries < 0:
+        raise Exception('--topology-build-retries can\'t be less than 0')
 
     def create_dir(path):
         if path:
@@ -271,22 +329,35 @@ def pytest_configure(config):
     from pyszn.injection import parse_attribute_injection
     injected_attr = None
     if injection_file is not None:
-
+        log.info('Processing attribute injection...')
+        start_time = time()
         # Get a list of all testing directories
         search_paths = [
-            abspath(arg) for arg in config.args if isdir(arg)
+            realpath(arg) for arg in config.args if isdir(arg)
         ]
 
         injected_attr = parse_attribute_injection(
             injection_file,
-            search_paths=search_paths
+            search_paths=search_paths,
+            ignored_paths=config.getini('norecursedirs'),
+            szn_dir=szn_dir
+        )
+        log.info(
+            'Attribute injection completed after {}s'
+            .format(time() - start_time)
         )
 
     # Create and register plugin
     config._topology_plugin = TopologyPlugin(
-        platform, plot_dir, plot_format.lstrip('.'),
-        nml_dir, injected_attr, log_dir,
-        parse_options(platform_options)
+        platform,
+        plot_dir,
+        plot_format.lstrip('.'),
+        nml_dir,
+        injected_attr,
+        log_dir,
+        szn_dir,
+        parse_options(platform_options),
+        build_retries
     )
     config.pluginmanager.register(config._topology_plugin)
 
@@ -345,6 +416,5 @@ __all__ = [
     'TopologyPlugin',
     'topology',
     'pytest_addoption',
-    'pytest_configure',
     'StepLogger'
 ]
